/* PropagationTree.java, v0.1, 23/02/2004
 * 
 * Copyright (c) 2003 Guillaume Arnaud, Inria Rocquencourt.  All rights reserved.
 * 
 */
package datastruct;

import request.PropagationRequest;

import java.util.Enumeration;
import java.util.Vector;
import java.awt.*;

import javax.swing.*;
/**
 * Implementation of a generic display tree.
 * 
 * @author Guillaume Arnaud
 */

public class PropagationTree extends GenericTree {
	
	/**
	 * Attributs.
	 */
	int mass=1; // Number of children
	boolean hidden=false; // Node must be hidden or not
	Point relativeCoordinate=new Point(0,0);	// Coordinates depending on the father
	Point3D relativeCoordinate3D=new Point3D(0,0,0);	// Coordinates 3D depending on the father 
	int widthDisplay=0; // Width of the subtree of this node
	int nbTotalPropagation=0;
	int nbActivate=0;
	int strahler=0;
	int massNbTotalPropagation=0;
	String externalRep="";
	boolean compactEdge=false;
	static boolean automaticHidden=false;
	int nbAloneChildren=0;
	public String label="";
	private DataStructRequest datastructrequest=new DataStructRequest();
	public int nodeNumber=0;
	boolean containsSolution=false,hasPropagation=false;
	PropagationRequest propagationRequest; // Requests if exists
	int repDisplayNode=0;

	/**
	* Constructors.
	*/ 
	public PropagationTree() {
		children = null;
		father = null;
	}

	public PropagationTree(String s) {
		children = null;
		father = null;
		this.setLabel(s);
	}
	
	public PropagationTree(boolean hp) {
		hasPropagation=hp;
	}

	public PropagationTree(String s, int nr) {
		nbTotalPropagation=nr;
		children = null;
		father = null;
		this.setLabel(s);
	}	
	public PropagationTree(String s, int nr,  boolean hp) {
		hasPropagation=hp;
		nbTotalPropagation=nr;
		children = null;
		father = null;
		this.setLabel(s);
	}
	public PropagationTree(String s, int nr,  boolean hp, PropagationRequest pr) {
		hasPropagation=hp;
		nbTotalPropagation=nr;
		children = null;
		father = null;
		this.setLabel(s);
		propagationRequest=pr;
	}
	public PropagationTree(String s, int nr, DataStructRequest ctr) {
		nbTotalPropagation=nr;
		children = null;
		father = null;
		datastructrequest=ctr;
		this.setLabel(s);
	}
	public PropagationTree(String s, int nr, DataStructRequest ctr, PropagationRequest propreq) {
		nbTotalPropagation=nr;
		children = null;
		father = null;
		datastructrequest=ctr;
		this.setLabel(s);
		propagationRequest =propreq;
	}
	public PropagationTree(String s, int nr, DataStructRequest ctr, boolean hp) {
		hasPropagation=hp;
		nbTotalPropagation=nr;
		children = null;
		father = null;
		datastructrequest=ctr;
		this.setLabel(s);
	}
	
	/**
	 * Hides tree.
	 */
	public void hideTree() {this.hidden=true;}

	/**
	 * Shows tree.
	 */
	public void showTree() {this.hidden=false;}
	
	/**
	 * Is an hidden tree?
	 */
	public boolean isHiddenTree() {return this.hidden;}

	/**
	 * Is a compact Edge?
	 */
	public boolean isCompactEdge() {return compactEdge;}

	/**
	 * Gets nbAloneChildren.
	 */
	public int getNbAloneChildren() {return nbAloneChildren;}

	/**
	 * Sets nodeNumber.
	 */
	public void setNodeNumber(int n) {nodeNumber=n;}
    
	/**
	 * Sets nodeNumber.
	 */
	public void setNodeNumber(String n) {nodeNumber=Integer.parseInt(n);}
    
	/**
	 * Gets datastrucrequest.
	 */
	public DataStructRequest getDataStructRequest() {return datastructrequest;}
	/**
	 * Gets propagationRequest.
	 */
	public PropagationRequest getPropagationRequest() {return propagationRequest;}
	/**
	 * Sets propagationRequest.
	 */
	public void setPropagationRequest(PropagationRequest pr) { propagationRequest=pr;}
	/**
	 * Gets mass.
	 */
	public int getMass(){return mass;}

	/**
	 * Gets nodeNumber.
	 */
	public int getNodeNumber(){return nodeNumber;}
	
	/**
	 * Gets strahler.
	 */
	public int getStrahler() {return strahler;}

	/**
	 * Gets repDisplayNode.
	 */
	public int getRepDisplayNode() {return repDisplayNode;}

	/**
	 * Sets label.
	 */
	public void setLabel(String l) { this.label=l;}
	
	/**
	 * Gets label.
	 */
	public String getLabel() { return this.label;}
	
	/**
	 * Gets the attribut point relativeCoordinate.
	 */
	
	public Point getRelativeCoordinate() {return this.relativeCoordinate;}

	/**
	 * Gets the attribut point relativeCoordinate3D.
	 */
	
	public Point3D getRelativeCoordinate3D() {return this.relativeCoordinate3D;}

	/**
	 * Gets nbTotalPropagation.
	 */
	public int getNbTotalPropagation() {return nbTotalPropagation;}
		
	/**
	 * Gets useles awakes rate.
	 */
	public int getUselessWakeup() {return datastructrequest.getUselessWakeup();}
	
	/**
	 * Gets nbActivate.
	 */
	public int getNbActivate() {return nbActivate;}
		
	/**
	 * Gets externalRep.
	 */
	public String getExternalRep() {return externalRep;}

	/**
	 * Sets externalRep.
	 */
	public void setExternalRep(Vector er) {
		Enumeration e;
        if(er==null){
            externalRep="no filterd constraint";
        }
        else{
            e=er.elements();
            externalRep="constraint:";
            while(e.hasMoreElements()){
                externalRep=externalRep+(String)e.nextElement()+"; ";
            }
        }
    }

	/**
	 * Gets children.
	 */
	public Vector getChildren() {return children;}
	/**
	 * Gets width display.
	 */
	public int getWidthDisplay() {return widthDisplay;}
	/**
	 * Sets nbTotalPropagation.
	 */
	public void setNbTotalPropagation(int nr) {nbTotalPropagation=nr;}
	
	/**
	 * Sets nbActivate.
	 */
	public void setNbActivate(int nr) {nbActivate=nr;}
	
	/**
	 * Sets automatic hide.
	 */
	public void setAutomaticHide(boolean b) {automaticHidden=b;}

	/**
	 * Is automatic hidden?
	 */
	public boolean isAutomaticHidden(){return automaticHidden;}

	/**
	 * Has propagation?
	 */
	public boolean hasPropagation(){return hasPropagation;}

	/**
	 * Sets datastructrequest.
	 */
	public void setDataStructRequest(DataStructRequest ctr) {
		datastructrequest=ctr;}

	/**
	 * Gets attribut father.
	 */
	public PropagationTree getFather() {
		if(this.father!=null) return (PropagationTree)this.father;
		else return null; }	

	/**
	 * Sets father.
	 */
	public void setFather(PropagationTree father){
		this.father=father;
	}
	
	/**
	 * Computes mass tree (number of children) and initializes
	 * each mass's node.
	 */
	public void compute_mass() { compute_mass_rec(this); return;}

	/**
	 * Computes mass tree (reccursive function).
	 */
	private int compute_mass_rec(PropagationTree t) { 
		Enumeration c;
		// an hidden node is considered as a leaf
		if(t.isHiddenTree()||!t.children()) {t.mass=1; return 1;}
		
		c=t.children.elements();		
		t.mass=1;
		while(c.hasMoreElements()){
			t.mass=t.mass+compute_mass_rec((PropagationTree)(c.nextElement()));
		}
		return t.mass;
	}

	/** 
	 * Adds child.
	 * Recomputes the mass node of ancestors.
	 */
	public void addChild(PropagationTree c) {
		if(children==null) children=new Vector();
		c.setFather(this);
		children.addElement(c);
		recomputeMassNode(c.getFather(),c.mass);	
	}

	/**
	 * Recomputes mass node of father.
	 */
	private void recomputeMassNode(PropagationTree t,int m) {
		  t.mass=t.mass+m;
		  if(t.father!=null) {recomputeMassNode(t.getFather(),m) ;}
		  return;
	}

	/**
	 * Go up solution existence information.
	 */
	public void setContainsSolution(){
		containsSolution=true;
		if(this.father!=null) {this.getFather().setContainsSolution();}
		return;
	}

	/**
	 * Contains a solution?
	 */
	public boolean containsSolution(){return containsSolution;}
	/**
	 * Recomputes the width display of each node considering their number of children.
	 */
/*	public void recomputeWidthDisplayStandard(PropagationTree t, int w) {
		Enumeration e;
		int nextWidth;
		

		t.widthDisplay=w;
		if(t.children()) { e=t.children.elements();
			nextWidth=w/t.children.size();
		}
		else return;

		while(e.hasMoreElements()){			
			recomputeWidthDisplayStandard((PropagationTree)e.nextElement(),nextWidth);
		}
		return;
	}*/

	/**
	 * Recomputes the standard display (width, depth and relative coordinates)
	 * of each node considering the number of children.
	 * w is the width of the whole tree and d the step of the depth 
	 * (unique for the whole tree).
	 */
	public void recomputeDisplayStandard(int w,int d, boolean is2D) {
		if(is2D) recomputeDisplayStandard(this,w,0,d,0);
		else compute3DTree(this,w,d,0,0,0,Math.PI/2);
		return;
	}

	/**
	 * Recomputes the standard display.
	 */
	private void recomputeDisplayStandard(PropagationTree t, int w, int ycurr, int d, int dxCurr) {
		Enumeration e;
		int nextWidth;
		
		if(t.isAutomaticHidden()&&t.children()&&2*t.children.size()>w) {t.hideTree();}
		t.widthDisplay=w;
		t.relativeCoordinate.setLocation(dxCurr,ycurr+d);
		
		if(!t.children() || t.isHiddenTree()) return;
		else {
			e=t.children.elements();
			if (t.children.size()==1) recomputeDisplayStandard((PropagationTree)e.nextElement(),w,(int)t.relativeCoordinate.getY(),d,dxCurr);
			else {
				nextWidth=w/t.children.size();
				dxCurr=dxCurr-w/2+nextWidth/2;
				while(e.hasMoreElements()) {
					recomputeDisplayStandard((PropagationTree)e.nextElement(),nextWidth,(int)t.relativeCoordinate.getY(),d,dxCurr);
					dxCurr=dxCurr+nextWidth;
				}
			}
		}
		return;
	}

	/**
	 * Computes coordinates in 3D.
	 */
	public void compute3DTree(PropagationTree t, int w, int d, int x, int y, int z, double angleinit){

		int r=w/2;
		int mass0=t.mass-1;
		t.relativeCoordinate3D.setLocation(x,y,z);
		//if(x==0 && y==0) System.out.println("New node="+x+","+y+","+z+".  Ray="+w);
		
		if(!t.children()||t.isHiddenTree()) return;
		int last=t.children.size()-1;
		if(t.children.size()==1) 
			compute3DTree((PropagationTree)t.children.firstElement(),w,d,x,y,z+d,angleinit);
		else {
			int znew=z+d,angleFactor=2,wnew;
			double angle,angleNew=0;
			Enumeration e=t.children.elements();
			int maxmass=0,maxmasspos=0,cpt=0; // position of the greatest element
			
			PropagationTree currentGDT;
			while(e.hasMoreElements()) {
				currentGDT=(PropagationTree)e.nextElement();
				int xm=1;
				if(!currentGDT.isHiddenTree()) {
					xm=currentGDT.getMass();
				}
				if(xm>maxmass) {
						maxmasspos=cpt;
						maxmass=xm;
					}
				cpt++;
			}
						
			int first=0;
			boolean endReached=false;
			cpt=maxmasspos;
			while(cpt!=maxmasspos || !endReached){
//				System.out.println(""+cpt);
				currentGDT=(PropagationTree)t.getChildren().elementAt(cpt);
				int mass=1;
				if(!currentGDT.isHiddenTree()) 
					mass=currentGDT.getMass();
				if(angleFactor==2 && mass>mass0-mass) {
					angleFactor=1;
					angle=Math.PI;
					mass0=mass0-mass;
				}
				else angle=angleFactor*Math.PI*mass/mass0;
				if(first==0) {
					first=1;
					angleNew=angleinit-angle/2;
				}

			/*	if(angle>=Math.PI && angleNew<1.5*Math.PI) {
					Enumeration e2=t.children.elements();
					PropagationTree currentGDT2=(PropagationTree)e2.nextElement();
					double angle2=0;
					angleNew=Math.PI/2;
					mass0=mass0-mass;
					while(currentGDT2!=currentGDT){
						angleNew=angleNew+angle2/2;
						wnew=(int)((double)w*Math.sin(angle2/2)/2);
						compute3DTree(currentGDT2,wnew,d,(int)(x+(double)r*Math.cos(angleNew)),(int)(y+(double)r*Math.sin(angleNew)),znew);
						angleNew=angleNew+angle2/2;
						currentGDT2=(PropagationTree)e2.nextElement();
						angle2=(Math.PI*currentGDT2.getMass())/(double)mass0;
					}					
					angleFactor=1;
					angle=Math.PI;
				}*/
				angleNew=angleNew+angle/2;
				wnew=(int)((double)w*Math.sin(angle/2)/2);
				compute3DTree(currentGDT,wnew,d,(int)(x+(double)r*Math.cos(angleNew)),(int)(y+(double)r*Math.sin(angleNew)),znew,angleNew);
				angleNew=angleNew+angle/2;
				if(cpt==last) {cpt=0;endReached=true;}
				else cpt++;
			}
		}
		return;
		
	}
	
	/**
	 * Recomputes the mass display (width, depth and relative coordinates)
	 * of each node considering the mass of children.
	 * w is the width of the whole tree and d the step of the depth 
	 * (unique for the whole tree).
	 */
	public void recomputeDisplayMass(int w,int d) {
		recomputeDisplayMass(this,w,0,d,0,this.mass);
		return;
	}
	/**
	 * Recomputes the mass display.
	 */
	private void recomputeDisplayMass(PropagationTree t, int w, int ycurr, int d, int dxCurr, int mass0) {
		Enumeration e;
		int currentWidth;
		
		if(t.isAutomaticHidden()&&t.children()&&2*t.children.size()>w) {t.hideTree();}
		t.widthDisplay=w;
		t.relativeCoordinate.setLocation(dxCurr,ycurr+d);
		
		if(!t.children() || t.isHiddenTree()) return;
		else {
			e=t.children.elements();
			if (t.children.size()==1)
				recomputeDisplayMass((PropagationTree)e.nextElement(),w,(int)t.relativeCoordinate.getY(),d,dxCurr,mass0-1);
			else {
				PropagationTree currentGDT;
				dxCurr=dxCurr-w/2;
				while(e.hasMoreElements()) {
					currentGDT=(PropagationTree)e.nextElement();
					if(currentGDT.isHiddenTree()||!currentGDT.children()) currentWidth=(int)((double)w/((double)(mass0-1)));
					else currentWidth=(int)(((double)(w*currentGDT.mass))/((double)(mass0-1)));
					dxCurr=dxCurr+currentWidth/2;
					recomputeDisplayMass(currentGDT,currentWidth,(int)t.relativeCoordinate.getY(),d,dxCurr,currentGDT.mass);
					dxCurr=dxCurr+currentWidth/2;
				}				
			}
		}
		return;
	}


	/**
	 * Hides the tree and recomputes the mass of ancestors.
	 */
	public void hideTreeRecomputation(){
		PropagationTree ancestor=this.getFather();
		int currentMass=this.mass;
		
		this.hideTree();
		while(ancestor!=null){
			ancestor.mass=ancestor.mass-currentMass+1;
			ancestor=ancestor.getFather();
		}
		return;	
	}
	/**
	 * Shows the tree and recomputes the mass of ancestors.
	 */
	public void showTreeRecomputation(){
		PropagationTree ancestor=this.getFather();
		int currentMass=this.mass;
		
		this.showTree();
		while(ancestor!=null){
			ancestor.mass=ancestor.mass+currentMass-1;
			ancestor=ancestor.getFather();
		}
		return;	
	}

	/**
	 * Changes the display state of the tree.
	*/
	public void changeDisplayState() {
		if(this.isHiddenTree())
			showTreeRecomputation();
		else
			hideTreeRecomputation();
		return;
	}

	/**
	 * Shows all nodes.
	 */
	public void showAll() { showAll(this); return; }

	/**
	 * Shows all children of g.
	 */
	private void showAll(PropagationTree g) {
		Enumeration e;
		g.showTree();

		if(g.children()) e=g.children.elements();
		else return;

		while(e.hasMoreElements()) {
			showAll((PropagationTree)e.nextElement()); }
		return;
	}
	
	/**
	 * Computes mass of number of reduction.
	 */
	public int computeMassNbTotalPropagation(PropagationTree t){
		Enumeration e;

		t.massNbTotalPropagation=t.getNbTotalPropagation();
		
		if(t.children()) {
			e=t.children.elements();
		}
		else return t.massNbTotalPropagation;
		
		while(e.hasMoreElements()){
			t.massNbTotalPropagation=t.massNbTotalPropagation+computeMassNbTotalPropagation((PropagationTree)e.nextElement()); }
		return t.massNbTotalPropagation;
	}

	/**
	 * Computes strahler coefficient, depending on nbTotalPropagation.
	 */
    	public void computeStrahler(PropagationTree t){
//		this.computeMassNbTotalPropagation(t);
		compute_strahler_nodes(t);
	}
	
	private int compute_strahler_nodes(PropagationTree t){
		Enumeration e;
		int strahlerA,strahlerB;
		int cste=1;		
		
	/*	if(t.isHiddenTree()){
			t.strahler=t.massNbTotalPropagation;
			return t.strahler;
		}*/
		
		if(t.children()) {
			e=t.children.elements();
			strahlerB=compute_strahler_nodes((PropagationTree)e.nextElement());
		}
		else{
			t.strahler=t.repDisplayNode;
			return t.strahler;
		}
		while(e.hasMoreElements()){
			strahlerA=compute_strahler_nodes((PropagationTree)e.nextElement());
			if(strahlerA!=strahlerB) {
				cste=2;
				if(strahlerA>strahlerB) strahlerB=strahlerA;
			}
		}
		t.strahler=t.repDisplayNode-cste+strahlerB+t.children.size();
		return t.strahler;
    }

	/**
	 * Computes the representation value of this node.
	 */
	public void computeRepInfo(boolean allred,Vector reduceFilterCheckBox, boolean allawake,Vector awakeFilterCheckBox){
		repDisplayNode=0;
		//repDisplayNode=t.getNbTotalPropagation();
		DataStructRequest dsr=this.getDataStructRequest();
		Enumeration e;
		/*if(dsr.hasRequest("reduce") && dsr.hasRequest("awake")) {
			repDisplayNode=dsr.getTotalEvents("awake");
			if(repDisplayNode!=0) {
				repDisplayNode=(100*(dsr.getTotalEvents("awake")-dsr.getTotalEvents("reduce")))/dsr.getTotalEvents("awake");
			}
		}*/
		if(allred) {
			repDisplayNode=dsr.getTotalEvents("reduce");
		}
		else if(reduceFilterCheckBox!=null) {
			e=reduceFilterCheckBox.elements();
			while(e.hasMoreElements()){
				JCheckBox cb=(JCheckBox)e.nextElement();
				if(cb.isSelected())
					repDisplayNode+=dsr.getConstraintFilterValue("reduce",cb.getText());
			}
		}
		if(allawake) {
			repDisplayNode+=dsr.getTotalEvents("awake");
		}
		else if(awakeFilterCheckBox!=null) {
			e=awakeFilterCheckBox.elements();
			while(e.hasMoreElements()){
				JCheckBox cb=(JCheckBox)e.nextElement();
				if(cb.isSelected())
					repDisplayNode+=dsr.getConstraintFilterValue("awake",cb.getText());
			}
		}
		if(this.children()) {
			e=this.children.elements();
			while(e.hasMoreElements()){
				((PropagationTree)e.nextElement()).computeRepInfo(allred,reduceFilterCheckBox, allawake,awakeFilterCheckBox);
			}
		}
		
	}

	/**
	 * Computes compacted edges of the current tree.
	 */
	public static void computeCompactEdge(PropagationTree t){
		PropagationTree c=t;
		Enumeration e;
		int nbAloneChildrenCurr=0;

		if(!t.children()) return;
		while(c.children()&&c.children.size()==1){
			nbAloneChildrenCurr++;
			c=(PropagationTree)c.children.firstElement();
		}
		if(nbAloneChildrenCurr>5) {
			t.compactEdge=true;
			t.nbAloneChildren=nbAloneChildrenCurr-1;
		}
		if(!c.children()) return;
		e=c.children.elements();
		while(e.hasMoreElements()){
			computeCompactEdge(((PropagationTree)e.nextElement()));
		}
		return;
	}
	

	/**
	 * Copies.
	 */
	public PropagationTree copy(){
		PropagationTree cloneTree=new PropagationTree(this.label,this.getNbTotalPropagation(),this.hasPropagation);
		Enumeration e;
		
		cloneTree.strahler=this.strahler;
		cloneTree.externalRep=this.externalRep;
		cloneTree.massNbTotalPropagation=this.massNbTotalPropagation;
		cloneTree.compactEdge=false;
		cloneTree.nbAloneChildren=this.nbAloneChildren;
		cloneTree.datastructrequest=this.datastructrequest.copy();
		cloneTree.nodeNumber=this.nodeNumber;
		cloneTree.containsSolution=this.containsSolution;
		cloneTree.repDisplayNode=this.repDisplayNode;
		cloneTree.propagationRequest=this.propagationRequest;
		
		
		if(!this.children()) {return cloneTree;}
		e=this.children.elements();
		while(e.hasMoreElements()){
			PropagationTree c=(PropagationTree)e.nextElement();
			cloneTree.addChild(c.copy());
		}
		return cloneTree;
	}

	/**
	 * Goes up in the tree. n is the number of ancestors to go up
	 * ( getFather()=up(1) ).
	 */
	public PropagationTree up(int n){
		PropagationTree ancestor=this;
		while(n!=0){
			if(ancestor.father()) ancestor=ancestor.getFather();
			else {
				System.out.println("Error in PropagationTree.up(int n) : no enough ancestors.");
				return null;
			}
			n--;
		}
		return ancestor;
	}

	/**
	 * Goes down in the tree taking the first child.
	 * n is the number of child to go down.
	 */
	public PropagationTree down(int n){
		PropagationTree c=this;
		while(n!=0){
			if(c.children()) c=(PropagationTree)c.children.firstElement();
			else {
				System.out.println("Error in PropagationTree.down(int n) : no enough ancestors.");
				return null;
			}
			n--;
		}
		return c;
	}
	
	///////////////////////////////
	// Inner Classes            ///
	///////////////////////////////
	
}
