/* file name  : gui/ControlPlay.java
 * authors    : Guillaume Arnaud
 * created    : lun 19 avr 2004 18:25:27 CEST
 * copyright  : 
 *
 * modifications:
 *
 */

package gui;

import parser.PropagationSolver;

import viewer.*;

import control.ThreadControl;

import java.io.PrintStream;
import java.io.PrintWriter;

import java.util.Vector;
import java.util.Enumeration;

import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Dimension;
import java.awt.Color;

import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;
import javax.swing.JTextField;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import fr.inria.oadymppac.parsing.event.Event;
import fr.inria.oadymppac.parsing.event.EventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.BreakPointEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.BreakPointEvent;
import fr.inria.oadymppac.parsing.gentra4cp.dtd.DTDBreakPointElement;
import fr.inria.oadymppac.parsing.gentra4cp.DefaultBreakPointEvent;

import fr.inria.oadymppac.parsing.gentra4cp.event.BackToEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.BackToEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewVariableEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewVariableEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.SolutionEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.SolutionEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.ChoicePointEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.ChoicePointEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewConstraintEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.NewConstraintEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.FailureEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.FailureEventListener;
import fr.inria.oadymppac.parsing.gentra4cp.event.ReduceEvent;
import fr.inria.oadymppac.parsing.gentra4cp.event.ReduceEventListener;

/** 
 * ControlPlay. Interface for controlling lecture of the trace.
 * 
 * @author Guillaume Arnaud
 * @version 1.2
 */
public class ControlPlay extends PavotFrame implements ActionListener, BreakPointEventListener,  BackToEventListener, NewVariableEventListener, SolutionEventListener, ChoicePointEventListener, NewConstraintEventListener, FailureEventListener, ReduceEventListener, Runnable{

    PropagationSolver propagationSolver;
    JButton startButton;
    JButton quitButton=new JButton("Quit Viewers");
    boolean start=false;
    PrintWriter socketOut;
    JTextField currentEvent=new JTextField(20);
    JMenuBar menuBar;
    JMenu menuView;
    JMenuItem pavotView;
    JMenuItem clpgui2dDomain;
    JMenuItem clpgui3dDomain;
    JMenuItem clpguiSearchTree;

    CTViewer ctview;
    CLPGUIDomain2D clpguiD2D;
    CLPGUIDomain3D clpguiD3D;
    CLPGUISearchTree clpguiST;

    Vector viewers;

    /** 
     * Constructor.
     * 
     * @param ps object which manages the parsing.
     */
    public ControlPlay(PropagationSolver ps, int dataType, PrintWriter so, Vector viewers){
        super();

        this.viewers=viewers;
        mainFrame.setTitle("Control");

        propagationSolver=ps;
        propagationSolver.addDefaultListener(this);

        // Main frame configuration
		mainFrame.setSize(200,200);
		mainFrame.getContentPane().setSize(200,200);
		mainFrame.getContentPane().setLayout(new BoxLayout(mainFrame.getContentPane(),BoxLayout.Y_AXIS));
        mainFrame.setBackground(Color.WHITE);

        // Menu 
        menuBar=new JMenuBar();
        menuView=new JMenu("Viewers");
        pavotView=new JMenuItem("Propagation Tree");pavotView.setEnabled(false);
        clpgui2dDomain=new JMenuItem("Domain 2D");clpgui2dDomain.setEnabled(false);
        clpgui3dDomain=new JMenuItem("Domain 3D");clpgui3dDomain.setEnabled(false);
        clpguiSearchTree=new JMenuItem("Search Tree");clpguiSearchTree.setEnabled(false);
        Enumeration enum=viewers.elements();
        int elt=0;
        while(enum.hasMoreElements()){
            PavotViewer pv=(PavotViewer)enum.nextElement();
            switch(pv.getType()){                
                case PavotViewer.PROPAGATIONTREE:   
                                                    ctview=(CTViewer)pv;
                                                    pavotView.addActionListener(new ActionListener(){
                                                        public void actionPerformed(ActionEvent e){
                                                            ctview.reshow();
                                                        }
                                                    });
                                                    pavotView.setEnabled(true);
                                                    break;                                                    
                case PavotViewer.CLPGUIDOMAIN2D:    clpguiD2D=(CLPGUIDomain2D)pv;
                                                    clpgui2dDomain.addActionListener(new ActionListener(){
                                                        public void actionPerformed(ActionEvent e){
                                                            clpguiD2D.reshow();
                                                        }
                                                    });
                                                    clpgui2dDomain.setEnabled(true);
                                                  break;
                case PavotViewer.CLPGUIDOMAIN3D:    clpguiD3D=(CLPGUIDomain3D)pv;
                                                    clpgui3dDomain.addActionListener(new ActionListener(){
                                                        public void actionPerformed(ActionEvent e){
                                                            clpguiD3D.reshow();
                                                        }
                                                    });
                                                  clpgui3dDomain.setEnabled(true);
                                                  break;
                case PavotViewer.CLPGUISEARCHTREE:    clpguiST=(CLPGUISearchTree)pv;
                                                      clpguiSearchTree.addActionListener(new ActionListener(){
                                                        public void actionPerformed(ActionEvent e){
                                                            clpguiST.reshow();
                                                        }
                                                    });
                                                  clpguiSearchTree.setEnabled(true);
                                                  break;
            }
            elt++;
        }
        
        menuView.add(pavotView);
        menuView.add(clpgui2dDomain);
        menuView.add(clpgui3dDomain);
        menuView.add(clpguiSearchTree);
        menuBar.add(menuView);
        mainFrame.setJMenuBar(menuBar);

        //
        mainFrame.getContentPane().add(Box.createRigidArea(new Dimension(0,10)));
        // Start button
        startButton=new JButton();
        socketOut=so;
        try{ 
                propagationSolver.addListener(propagationSolver.addElement(DTDBreakPointElement.BREAKPOINT,DefaultBreakPointEvent.class),this);
                System.out.println("[ControlPlay] addBreakPointListener success");
            }
            catch(Exception e){System.out.println("[ControlPlay] addBreakPointListener fails");}
            
        if(dataType==UISolverSelection.SOCKET){
            startButton.setText("Connected...");
            startButton.setEnabled(false);
        }
        else {
            startButton.setText("Read trace");
            startButton.setEnabled(true);}
        startButton.addActionListener(this);
		mainFrame.getContentPane().add(startButton);

        //
        mainFrame.getContentPane().add(Box.createRigidArea(new Dimension(0,10)));
        mainFrame.getContentPane().add(new JSeparator(SwingConstants.HORIZONTAL));
        mainFrame.getContentPane().add(Box.createRigidArea(new Dimension(0,10)));
        // Quit button
        quitButton.addActionListener(this);
		mainFrame.getContentPane().add(quitButton);

        // Current event text
        mainFrame.getContentPane().add(Box.createRigidArea(new Dimension(0,10)));
        mainFrame.getContentPane().add(new JSeparator(SwingConstants.HORIZONTAL));
        currentEvent.setEditable(false);
        currentEvent.setText("Start");
        mainFrame.getContentPane().add(currentEvent);
        
        //
        mainFrame.getContentPane().add(Box.createRigidArea(new Dimension(0,10)));
        // Initializes view
		mainFrame.setVisible(true);
		mainFrame.show();        

        // If connection by socket start immediatly
        if(dataType==UISolverSelection.SOCKET){

        // Providtruee
        System.out.println("toto");
        socketOut.println("RESET\r");
        socketOut.println("ADD search:failure backTo: true do current(chrono,node,port)\r");
        socketOut.println("ADD dom:choicePoint solution:true do current(chrono,node,port,named_vars,full_dom)\r");
        socketOut.println("ADD propag:reduce awake: true do current(chrono,cident)\r");
        socketOut.println("ADD newCstr:newConstraint:true do current(chrono,cident,cinternal)\r");
        socketOut.println("ADD newVar:newVariable:true do current(chrono,vident,vname)\r");
        socketOut.println("GO\r");
        socketOut.flush();

        propagationSolver.start();
        }
    }

    public void run(){ System.out.println("[ControlPlay] run");
    }
        
    public boolean isStarted(){return start;}

    public synchronized void actionPerformed(ActionEvent e){
        if(e.getSource()==startButton){
            start=true;
            propagationSolver.start();
            startButton.setEnabled(false);
        }
        else if (e.getSource()==quitButton){
            System.exit(0);
            System.out.println("[ControlPlay] Try interrupt");
            try{ propagationSolver.getThreadControl().myInterrupt(); }
            catch(InterruptedException ex) {System.out.println("[ControlPlay] interrupt fails 1");}
            catch(IllegalMonitorStateException ex2) {System.out.println("[ControlPlay] interrupt fails 2");}
            System.out.println("[ControlPlay] interrupt finished");
        }
    }
    
    public void windowClosing(WindowEvent e){
        System.exit(0);
    }

    public void eventProcess(BreakPointEvent e){
        System.out.println("[ControlPlay] breakpoint");
      /*  if(socketOut!=null) {
            socketOut.writeToTracer("GO");socketOut.flush();}*/
 //       else System.out.println("[ControlPlay] BreakPoint Error: socket null");
    }

    public void eventProcess(BackToEvent e){
        currentEvent.setText("back to : chrono="+e.getChrono());
    }
    public void eventProcess(NewVariableEvent e){
        currentEvent.setText("new var : chrono="+e.getChrono());
    }
    public void eventProcess(SolutionEvent e){
        currentEvent.setText("solution : chrono="+e.getChrono());
/*        System.out.println("[ControlPlay] Solution: GO!!!!!!!!!");
        if(socketOut!=null) {
            socketOut.writeToTracer("GO");socketOut.flush();}          */
    }
    public void eventProcess(ChoicePointEvent e){
        currentEvent.setText("choicepoint : chrono="+e.getChrono());
 /*       System.out.println("[ControlPlay] Choice point: GO!!!!!!!!!");
        if(socketOut!=null) {
            socketOut.writeToTracer("GO");socketOut.flush();}        */
    }
    public void eventProcess(NewConstraintEvent e){
        currentEvent.setText("new cstr : chrono="+e.getChrono());
 /*      System.out.println("[ControlPlay] NewConstraintEvent: GO!!!!!!!!!");
        if(socketOut!=null) {
            socketOut.writeToTracer("GO");socketOut.flush();}                */
    }
    public void eventProcess(FailureEvent e){
        currentEvent.setText("failure : chrono="+e.getChrono());
    }
    public void eventProcess(ReduceEvent e){
/*       System.out.println("[ControlPlay] Reduce: GO!!!!!!!!!");
        if(socketOut!=null) {
            socketOut.writeToTracer("GO");socketOut.flush();}                        */
    }    
    
    public void eventProcess(Event e){
    }
}
