/*
 * $Id: UIManager.java, v0.0 2004/01/20$
 */
package gui;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

import javax.swing.*;

import request.PropagationRequest;

import viewer.ViewerConstants;

public class UIManager implements ActionListener {
	// GUI Variables
	JFrame mainFrame=new JFrame();
	JCheckBox ctbox=new JCheckBox("Propagation tree");
	JCheckBox stclpgui=new JCheckBox("CLPGUI search tree (3D)");
	UIOneQuery awakeQuery =new UIOneQuery(new  String("awake           "),this,true,450,20);
	UIOneQuery reduceQuery =new UIOneQuery(new String("reduce           "),this,true,450,20);
	JButton ok=new JButton("Run viewer(s)");
	
	// Application variables 
	boolean isfinish=false;
	Vector viewers=new Vector();
	PropagationRequest propReq;
	JLabel viewersLabel = new JLabel("Select one or several viewer:",SwingConstants.LEFT);
	JLabel filtersLabel = new JLabel("Select filter (for propagation viewers):",SwingConstants.LEFT);

	public UIManager(PropagationRequest pr){
        
		propReq=pr;
		viewers.add(Boolean.FALSE); viewers.add(Boolean.FALSE); viewers.add(Boolean.FALSE); viewers.add(Boolean.FALSE); viewers.add(Boolean.FALSE);
		// initialize the main frame
		mainFrame.setTitle("Viewers selection and filters");
		mainFrame.setSize(500,250);
		mainFrame.getContentPane().setSize(500,250);
		mainFrame.getContentPane().setLayout(new BoxLayout(mainFrame.getContentPane(),BoxLayout.Y_AXIS));
		mainFrame.setResizable(true);
		mainFrame.getRootPane().setMinimumSize(new Dimension(500,250));
        mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// Viewers Selection
		viewersLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		mainFrame.getContentPane().add(viewersLabel );
		mainFrame.getContentPane().add( Box.createRigidArea(new Dimension(0,5)));
		ctbox.setPreferredSize(new Dimension(450,20));
		ctbox.setMaximumSize(new Dimension(450,20));
		ctbox.setAlignmentX(Component.LEFT_ALIGNMENT);
		ctbox.addActionListener(this);
		mainFrame.getContentPane().add(ctbox);		
		stclpgui.setPreferredSize(new Dimension(450,20));
		stclpgui.setMaximumSize(new Dimension(450,20));
		stclpgui.setAlignmentX(Component.LEFT_ALIGNMENT);
		stclpgui.addActionListener(this);
		mainFrame.getContentPane().add(stclpgui);
		// Separation
		mainFrame.getContentPane().add( Box.createRigidArea(new Dimension(0,5)));
		mainFrame.getContentPane().add(this.createSeparator(SwingConstants.HORIZONTAL,1000,10));
		mainFrame.getContentPane().add( Box.createRigidArea(new Dimension(0,5)));
		// Filters Selection
		filtersLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		mainFrame.getContentPane().add(filtersLabel);
		mainFrame.getContentPane().add( Box.createRigidArea(new Dimension(0,5)));
//		reduceQuery.setBorder(BorderFactory.createLineBorder(Color.black));
		reduceQuery.setActivate(false);
		reduceQuery.setAlignmentX(Component.LEFT_ALIGNMENT);
		mainFrame.getContentPane().add(reduceQuery);	
		mainFrame.getContentPane().add( Box.createRigidArea(new Dimension(0,8)));
//		awakeQuery.setBorder(BorderFactory.createLineBorder(Color.black));
		awakeQuery.setActivate(false);
		awakeQuery.setAlignmentX(Component.LEFT_ALIGNMENT);
		mainFrame.getContentPane().add(awakeQuery);
		// Separation
		mainFrame.getContentPane().add( Box.createRigidArea(new Dimension(0,5)));
		mainFrame.getContentPane().add(this.createSeparator(SwingConstants.HORIZONTAL,1000,10));
		mainFrame.getContentPane().add( Box.createRigidArea(new Dimension(0,10)));
		// Run viewers
		ok.addActionListener(this);
		mainFrame.getContentPane().add(ok);

		mainFrame.getContentPane().add(Box.createVerticalGlue());
		mainFrame.setVisible(true);
		mainFrame.show();
		
	}

	/**
	 * create separator.
	 */
	private JSeparator createSeparator(int axis,int w,int h){
		JSeparator separator=new JSeparator(axis);
		separator.setPreferredSize(new Dimension(w,h));
		separator.setMaximumSize(new Dimension(w,h));
		separator.setAlignmentX(Component.LEFT_ALIGNMENT);
		return separator;
	}
	
	/**
	 * Action events.
	 */
	public void actionPerformed(ActionEvent e){
		if(e.getSource()==ok && (ctbox.isSelected()||stclpgui.isSelected())){
			// Run viewers and finish this frame
			if(reduceQuery.isSelected() ) {
				propReq.addRequest("reduce",reduceQuery.hasFilters(),reduceQuery.getFilters()); }
			if(awakeQuery.isSelected()) {
				propReq.addRequest("awake",awakeQuery.hasFilters(),awakeQuery.getFilters()); }
			isfinish=true;
		}
		else {	
			// 
			if(ctbox.isSelected()) {
				viewers.set(ViewerConstants.PAVOT,Boolean.TRUE);
				reduceQuery.setActivate(true);
				awakeQuery.setActivate(true);
			}
			else {
				viewers.set(ViewerConstants.PAVOT,Boolean.FALSE);
				reduceQuery.setActivate(false);
				awakeQuery.setActivate(false);
			}
			if(stclpgui.isSelected()) viewers.set(ViewerConstants.CLPGUI,Boolean.TRUE);
			else viewers.set(ViewerConstants.CLPGUI,Boolean.FALSE);
		}
	}

	/**
	 * Refreshes.
	 */
	public void refresh() {
		mainFrame.show();
	}
	/**
	 * Gets viewers.
	 */
	public Vector getViewers() { return viewers;}

	/**
	 * Is the user click ok button?
	 */
	public boolean isFinish() {return isfinish;}

	/**
	 * Dispose the main frame.
	 */
	public void dispose(){
		mainFrame.dispose();
	}

}
