/* file name  : viewer/CLPGUIDomain2D.java
 * authors    : Guillaume Arnaud
 * created    : jeu 01 avr 2004 16:11:26 CEST
 * copyright  : 
 *
 * modifications: Class from the inner class Domains2D of CLPGUI.java 
 * 
 */
package viewer;

import java.awt.*;
import java.awt.event.*;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import datastruct.clpgui.Domains2D;

import fr.inria.oadymppac.parsing.info.ValuesInfo;
import fr.inria.oadymppac.parsing.info.RangeInfo;
import fr.inria.oadymppac.parsing.event.Info;
/** 
 * Represents in 2D a finite domain for each variable. 
 * 
 * @author Franois Fages, Guillaume Arnaud
 * @version 1.0
 */
public class CLPGUIDomain2D extends Frame implements WindowListener, PavotViewer{

	int type=PavotViewer.CLPGUIDOMAIN2D;

	/** 
	 * Width of the frame. Default value=500
	 */
	int width=500;
	
	/** 
	 *  Height of the frame. Default value={@value}
	 */
	int height=500;
	
	/** 
	 * Width of a square of domain.
	 */
	int di;
    
    /** 
     * Height of a square of domain.
     */
    int dj;
	
    /** 
     * Data structure which contains domain information.
     */
    Domains2D domains2D;

	/** 
	 * Graphics for double buffering.
	 */
	Graphics gMem;  
	
	/** 
	 * Image for double buffering.
	 */
	Image memory;

	/** 
	 * Constructor.
	 */
	public CLPGUIDomain2D(Domains2D domains2D) {
        super();
        this.setTitle("CLPGUI Finite Domain 2D Viewer");
        if(domains2D.getInfoDomain()==Domains2D.SIZEDOMAIN) return;
        this.addWindowListener(this);
        this.domains2D=domains2D;
	    setSize(width,height);
	    init();
        this.show();
	}
	

	/** 
	 * Initializes the frame.
	 */
	private void init() {
	    di=(height-60)/(domains2D.getNbVar()+1);
	    if (di<1) di=1;
	    dj=(width-60)/(domains2D.getVMax()-domains2D.getVMin()+1);
	    if (dj<1) dj=1;
	}

	/** 
	 * Resizes the frame.
	 */
	private void resize() {
	    memory = createImage(width,height);
	    gMem = memory.getGraphics();
	    gMem.setFont(new Font("monospaced",Font.PLAIN,12));
	    clear();
	}

	/** 
	 * Clears the frame.
	 */
	private void clear() {
	    init();
	    gMem.setColor(Color.white);
	    gMem.fillRect(0, 0, width, height);
	    gMem.setColor(Color.lightGray);
	    for (int v=0;v<domains2D.getNbVar();v++) {
		gMem.drawString(domains2D.getName(v),5,60+v*di+di);
		gMem.drawLine(50,60+v*di,50+(domains2D.getVMax()-domains2D.getVMin()+1)*dj,60+v*di);
	    }
	    gMem.drawLine(50,60+domains2D.getNbVar()*di,50+(domains2D.getVMax()-domains2D.getVMin()+1)*dj,60+domains2D.getNbVar()*di);
	    for (int v=domains2D.getVMin();v<=domains2D.getVMax();v++) {
		gMem.drawString(""+v,50+(v-domains2D.getVMin())*dj,45);
		gMem.drawLine(50+(v-domains2D.getVMin())*dj,60,50+(v-domains2D.getVMin())*dj,60+domains2D.getNbVar()*di);
	    }
	    gMem.drawLine(50+(domains2D.getVMax()+1-domains2D.getVMin())*dj,60,50+(domains2D.getVMax()+1-domains2D.getVMin())*dj,60+domains2D.getNbVar()*di);
	    gMem.setColor(Color.red);
	}

	/** 
	 * Fills the square of the variable index v and value d.
	 * 
	 * @param v int of index of variable.
	 * @param d int of value of variable.
	 */
	private void inDomain(int v, int d) {
	    gMem.fillRect(50+(d-domains2D.getVMin())*dj,60+v*di,dj,di);
	}
    
	/** 
	 * Fills the square of the variable index v and value d.
	 * 
	 * @param v int of index of variable.
	 * @param ri 
	 */
	private void inDomain(int v, RangeInfo ri) {
       /* for(i=ri.getFrom();i==ri.getTo();i++){
	        gMem.fillRect(50+(i-domains2D.getVMin())*dj,60+v*di,dj,di);
        }*/
        inDomain(v,ri.getFrom(),ri.getTo());
	}
	/** 
	 * Fills the square of the variable index v and value d.
	 * 
	 * @param v int of index of variable.
	 * @param ri 
	 */
	private void inDomain(int v, ValuesInfo vi) {
        int i;
        int[] values=vi.getValues();
        for(i=0;i<values.length;i++){
	        gMem.fillRect(50+(values[i]-domains2D.getVMin())*dj,60+v*di,dj,di);
        }
	}
	/** 
	 * Fills the squares of the variable index v between value min and max.
	 * 
	 * @param v int of index of variable.
	 * @param min int of minimum value of variable.
	 * @param max int of maximum value of variable.
	 */
	private void inDomain(int v, int min, int max) {
	    gMem.fillRect(50+(min-domains2D.getVMin())*dj,60+v*di,(max-min+1)*dj,di);
	}

	public synchronized void paint(Graphics g) {
	    if (width!=getSize().width || height!=getSize().height) {
		width=getSize().width;
		height=getSize().height;
		memory=null;
	    }
	    if (memory==null) resize();
	    g.drawImage(memory,0,0,this);
	}

    /** 
     * Refreshes the frame. Reads each values of each variables of the current {@link #domains2D}
     * and calls {@link #inDomain/2}.
     */
    public void refresh() {
        int v;
        int d;
        this.clear();
        for(v=0;v<domains2D.getNbVar();v++){
            Vector domain=(Vector)domains2D.getDomains().get(domains2D.getIdent(v));
            Enumeration enu=domain.elements();
            while(enu.hasMoreElements()){   
                Object o=enu.nextElement();
                try { this.inDomain(v,(ValuesInfo) o); }
                catch(Exception ex1) {
                    try { this.inDomain(v,(RangeInfo) o); }
                    catch(Exception ex2) {//System.out.println("Error in domain");
                    }
                }
            }
        }
        this.repaint();
    }
    
    public Container getContentContainer(){return this;}

	public void windowClosing(WindowEvent e) { this.dispose();}
	public void windowClosed(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}

    public int getType(){return type;}

    public void reshow(){
        this.show();
    }   

}

