/* file name  : viewer/CLPGUIDomain3D.java
 * authors    : Guillaume Arnaud
 * created    : ven 09 avr 2004 14:49:12 CEST
 * copyright  : 
 *
 * modifications:
 *
 */
package viewer;

import datastruct.clpgui.*;

import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.Container;

import java.awt.Canvas;
import java.awt.CheckboxMenuItem;
import java.awt.MenuBar;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Font;
import java.awt.Color;


public class CLPGUIDomain3D extends CLPGUICanvas3D implements ItemListener, PavotViewer {
	int type=PavotViewer.CLPGUIDOMAIN3D;

	Menu options=new Menu("Options");
    CheckboxMenuItem backtrackTrace = new CheckboxMenuItem("Trace backtracking", false);

    Domains2D domains2D;
    Domains3D domains3D;
    SearchTree highlight;

    static final Color domainsColor=new Color(100,100,250);
    static final Color domainsLightColor=new Color(150,150,250);
    static final int charSize=8;

	int scaleX, scaleY, scaleZ;
	int kv, kt, ks, rv, rt, rs;
	int varHighlight=-1;
	
	int v,k,d;
    
    CLPGUIFrame3D windowDomain3D;

	public CLPGUIDomain3D(){
	    super();

        type=CLPGUIDOMAIN3D;
	    options.add(backtrackTrace);
	    backtrackTrace.addItemListener(this);
	    menuBar.add(options);

        windowDomain3D=new CLPGUIFrame3D(this);
 	    windowDomain3D.setTitle("CLPGUI Finite Domains 3D Viewer");
	    windowDomain3D.addWindowListener(windowDomain3D);
        windowDomain3D.show();
        this.setVisible(true);
	    clear();
	}
    
	public CLPGUIDomain3D(Domains2D domains2D,Domains3D domains3D){
	    super();
        type=PavotViewer.CLPGUIDOMAIN3D;
	    options.add(backtrackTrace);
	    backtrackTrace.addItemListener(this);
	    menuBar.add(options);
        this.domains2D=domains2D;
        this.domains3D=domains3D;

        windowDomain3D=new CLPGUIFrame3D(this);
 	    windowDomain3D.setTitle("CLPGUI Finite Domains 3D Viewer");
	    windowDomain3D.addWindowListener(windowDomain3D);
        windowDomain3D.show();
        this.setVisible(true);
	    clear();
	}
    
	void clear() {
	    kv=40;
	    kt=40;
	    ks=20;
	    center();	    
	}

	void center() {
	    super.center();
	    oX=width/2-50;
	    oY=10+height/3;
	    aZ=-pi4;
	    aY=pi6;
	    initRotate();
	    repaint();
	}

	void resized(int width, int height) {
	    super.resized(width,height);
	    oX=width/2-30;
	    oY=10+height/3;
	    scaleX=width/2+100;
	    scaleY=width/2+100;
	    scaleZ=height/3;
	    autoscale();
	    refresh=true;
	}
	
	void autoscale() {
	    if (kv*domains2D.getNbVar()<width/3) kv=40;
	    if (ks*domains3D.getMaxSize()<height/4) ks=20;
	    if (kt*domains3D.getTime()<width/3) kt=20;

	    if (kv*domains2D.getNbVar()>width/2 || kv==1) kv=width/(2*(domains2D.getNbVar()+1));
	    if (ks*domains3D.getMaxSize()>height/3 || ks==1) ks=height/(3*(domains3D.getMaxSize()+1));
	    if (kt*domains3D.getTime()>width/2 || kt==1) kt=width/(2*(domains3D.getTime()+1));
	    if (kv<1) kv=1;
	    if (ks<1) ks=1;
	    if (kt<1) kt=1;
	}

	public synchronized void mouseMoved(MouseEvent e)
	{
	    double x=x(e.getX(),e.getY())/kv;
	    double y=y(e.getX(),e.getY())/kt;

        refresh=true;
	    
	    if (y>0 & y<domains3D.getTime()+10 & x>0 & x<domains2D.getNbVar()) {
		if (y>domains3D.getTime()) {
		    if (varHighlight!=(int)x) {
			refresh=true;
			varHighlight=(int)x;
			domains3D.setInfo("");
		    }
		}
		else {
		    varHighlight=-1;
		    domains3D.setInfo("");
		}
		if (x>domains2D.getNbVar()-1 & y<domains3D.getTime()) {
		    highlight=(domains3D.getExecutedGoals())[(int)y]; 
		    refresh=true;
		}
		else if (highlight!=null) {
		    refresh=true;
		    highlight=null;
		}
	    }
	    else {
		varHighlight=-1;
		domains3D.setInfo("");
	    }
	    paint(getGraphics());
	}
	
    public void itemStateChanged(ItemEvent e){
        String s=e.getItem().toString();
        if(s.equals("Trace backtracking")) domains3D.setBacktrackTrace(backtrackTrace.getState());
    }
    
	public synchronized void drawAll()
	{
		gMem.setColor(Color.gray);
		gMem.drawString("0",i(-10,0,0),j(-10,0,0));
		gMem.drawLine(i(0,0,0),j(0,0,0),i(scaleX,0,0),j(scaleX,0,0));
		gMem.drawLine(i(0,0,0),j(0,0,0),i(0,scaleY,0),j(0,scaleY,0));
		gMem.drawLine(i(0,0,0),j(0,0,0),i(0,0,scaleZ),j(0,0,scaleZ));
		gMem.drawString(domains2D.getNbVar()+" variables",i(scaleX,0,0),j(scaleX,0,0));
		gMem.drawString("time "+domains3D.getTime(),i(0,scaleY,0),j(0,scaleY,0));
		gMem.drawString("domain size "+domains3D.getMaxSize(),i(0,0,scaleZ),j(0,0,scaleZ));

		for (int t=0; t<domains3D.getTime(); t++) {
		    rt=kt*t;

		    Color col;            
		    if (domains3D.ancestor((domains3D.getExecutedGoals())[t]))
			col=domainsColor;
		    else
			col=domainsLightColor;

		    for (int v=0; v<domains2D.getNbVar(); v++) {
                rv=kv*v;
                rs=ks*((domains3D.getDomains2D(t)).getSize(v));
                // top facet
                x[0]=i(rv+kv,rt,rs);
                y[0]=j(rv+kv,rt,rs);
                x[1]=i(rv,rt,rs);
                y[1]=j(rv,rt,rs);
                x[2]=i(rv,rt+kt,rs);
                y[2]=j(rv,rt+kt,rs);
                if ( v<domains2D.getNbVar()-1 && t<domains3D.getTime()-1 && rs<=ks*domains3D.getDomains2D(t).getSize(v+1) && rs<=ks*domains3D.getDomains2D(t+1).getSize(v)) {
                    // top facet only
                    x[3]=i(rv+kv,rt+kt,rs);
                    y[3]=j(rv+kv,rt+kt,rs);
                    gMem.setColor(col);
                    gMem.fillPolygon(x,y,4);
                    gMem.setColor(Color.black);
                    gMem.drawPolygon(x,y,4);
                }
                else {
                    // left and right facets added
                    x[3]=i(rv,rt+kt,0);
                    y[3]=j(rv,rt+kt,0);
                    x[4]=i(rv+kv,rt+kt,0);
                    y[4]=j(rv+kv,rt+kt,0);
                    x[5]=i(rv+kv,rt,0);
                    y[5]=j(rv+kv,rt,0);
                    gMem.setColor(col);
                    gMem.fillPolygon(x,y,6);
                    gMem.setColor(Color.black);
                    gMem.drawPolygon(x,y,6);
                    x[1]=i(rv+kv,rt+kt,rs); // missing edges
                    y[1]=j(rv+kv,rt+kt,rs);
                    gMem.drawLine(x[1],y[1],x[4],y[4]);
                    gMem.drawLine(x[1],y[1],x[0],y[0]);
                    gMem.drawLine(x[1],y[1],x[2],y[2]);
                }
                int[] domainVec=(domains3D.getDomains2D(t)).domainToVector(v);
                if(domainVec!=null){
                    k=0;
                    d=(domains3D.getDomains2D(t)).getSize(v);
                    for(int i=0;i<d;i++){                    
                        drawValue(""+domainVec[i]);
                        k++;
                    }
                }
		    }
		    drawGoalName((domains3D.getExecutedGoals())[t],kv*domains2D.getNbVar(),kt*t+kt,0);
		}
	}

	public synchronized void drawGoalName(SearchTree t, double x, double y, double z) {
	    
	    if (t.property.endsWith("success")) {
		gMem.setColor(Color.red);
		gMem.drawString("+",i(x,y,z)-charSize, j(x,y,z));
	    }
	    
	    if (t==domains3D.getCurrent()) {
		gMem.setColor(Color.blue);
	    }
	    else if (t==domains3D.getRoot()) {
		gMem.setColor(Color.green);
	    }
	    else if (t==highlight || t.property.startsWith("show")) {
		gMem.setColor(Color.red);
	    }
	    else return;
	    
	    int l; if (zoomX>0) l=t.label.length()*charSize; else l=charSize;
	    if (t.property.endsWith("success")) 
		gMem.drawString(t.label, i(x,y,z)-l-charSize, j(x,y,z));
	    else
		gMem.drawString(t.label, i(x,y,z)-l, j(x,y,z));
	}
    
	public synchronized void eraseNames()
	{
	    gMem.setColor(Color.white); // erases variable names and previous time
	    for (int v=0; v<domains2D.getNbVar(); v++) {
		gMem.drawString(domains2D.getName(v),i(kv*(v+1),kt*domains3D.getTime()+20,0),j(kv*(v+1),kt*domains3D.getTime()+20,0));
	    }
	    gMem.drawString("domains3D.getTime() "+(domains3D.getTime()-1),i(0,scaleY,0),j(0,scaleY,0));
	    gMem.drawString("time "+(domains3D.getTime()+1),i(0,scaleY,0),j(0,scaleY,0));

	    gMem.setColor(Color.gray);
	    gMem.drawString("time "+domains3D.getTime(),i(0,scaleY,0),j(0,scaleY,0));

	}

	public void parameter(int var)
	{
	    v=var;
	    d=domains2D.getSize(v);
	    rt=kt*domains3D.getTime();
	    rv=kv*v;
	    rs=ks*d;
	}

	public synchronized void drawDomainVar() 
	{
	    // top facet
	    x[0]=i(rv+kv,rt,rs);
	    y[0]=j(rv+kv,rt,rs);
	    x[1]=i(rv,rt,rs);
	    y[1]=j(rv,rt,rs);
	    x[2]=i(rv,rt+kt,rs);
	    y[2]=j(rv,rt+kt,rs);
	    x[3]=i(rv,rt+kt,0);
	    y[3]=j(rv,rt+kt,0);
	    x[4]=i(rv+kv,rt+kt,0);
	    y[4]=j(rv+kv,rt+kt,0);
	    x[5]=i(rv+kv,rt,0);
	    y[5]=j(rv+kv,rt,0);
	    gMem.setColor(domainsColor);
	    gMem.fillPolygon(x,y,6);
	    gMem.setColor(Color.black);
	    gMem.drawPolygon(x,y,6);
	    x[1]=i(rv+kv,rt+kt,rs); // missing edges
	    y[1]=j(rv+kv,rt+kt,rs);
	    gMem.drawLine(x[1],y[1],x[4],y[4]);
	    gMem.drawLine(x[1],y[1],x[0],y[0]);
	    gMem.drawLine(x[1],y[1],x[2],y[2]);
	}

	public synchronized void drawValue(String z)
	{
	    gMem.setColor(Color.white);
	    gMem.drawString(z, i(rv+kv/2, rt+kt, rs*k/d+rs/(2*d)), j(rv+kv/2, rt+kt, rs*k/d+rs/(2*d)));
	}

	public synchronized void drawNames()
	{
	    gMem.setColor(Color.black);
	    for (int v=0; v<domains2D.getNbVar(); v++) {
		gMem.drawString(domains2D.getName(v),i(kv*(v+1),kt*domains3D.getTime()+20,0),j(kv*(v+1),kt*domains3D.getTime()+20,0));
	    }
	    if (varHighlight>=0 & varHighlight<domains2D.getNbVar()) {
		gMem.setColor(Color.red);
		if (!domains3D.getInfo().startsWith(domains2D.getName(varHighlight)+"=")) {
		    domains3D.setInfo(domains2D.getName(varHighlight)+"="+domains2D.domainToString(varHighlight));
		}
		gMem.drawString(domains3D.getInfo(),i(kv*(varHighlight+1),kt*domains3D.getTime()+20,0),j(kv*(varHighlight+1),kt*domains3D.getTime()+20,0));
	    }
	}


	public synchronized void paint(Graphics g)     
	{
	    if (kt>1 && kt*domains3D.getTime()>width) refresh=true;

	    super.paint(g);

	    if (refresh) {
		autoscale();
		drawAll();
		refresh=false;
	    }

	    drawNames();

	    g.drawImage(memory,0,0,this);
	}	    

    public void refresh(){        
		for (int v=0; domains3D.getTime()>0 && v<domains2D.getNbVar(); v++) {
            int d= domains3D.getDomains2D(domains3D.getTime()-1).getSize(v);
		    if (d>domains3D.getMaxSize()) domains3D.setMaxSize(d);
		    if (domains3D!=null) {
			this.parameter(v);
			this.drawDomainVar();
		    }
            
		}

        refresh=true;
        repaint();
    }

    public Container getContentContainer(){return windowDomain3D;}

    public SearchTree getHighLight(){return highlight;}

    public int getType(){return type;}
    
    public void reshow(){
        windowDomain3D.show();
    }	
}


