// A Graphical User Interface for Constraint Logic Programming
// http://contraintes.inria.fr/~fages/CLPGUI
// Copyright (C) 2001, 2002, INRIA Francois Fages
// License LGPL-2.1


// CLPGUISearchTree is provided by CLPGUI and has been adapted for using the
// 3D search tree.

package viewer;

import datastruct.clpgui.SearchTree;
import datastruct.clpgui.SearchTreeList;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

	


    /////////////////////////
    // Search Tree viewers //
    /////////////////////////

public class CLPGUISearchTree extends CLPGUICanvas3D implements ItemListener, ActionListener, PavotViewer
    {
	int type=PavotViewer.CLPGUISEARCHTREE;
	static final int NEAR=25;
	SearchTree fRoot, sRoot, highlight, root;//fcurrent

	Menu trees;
	CheckboxMenuItem freezeTree,showGoals, fsp, fw, tm3d;
	Menu nodes;
	MenuItem restoreHidden, removeRecomputed, recomputeFast, recomputeSideEffects, hideSubtree, showSubtree, showCurrentGoal;
	boolean noHide, noRecompute;

	boolean drawCenteredStrings=true;
	SearchTree goalHighLight=null;

	int im,jm;
	boolean clicked=false;
	boolean showCompleteGoal=false;
	String nearString;
	static final int charSize=8;
    CLPGUIFrame3D windowSearchTree3D;

	public CLPGUISearchTree(SearchTree st)
	{
	    super();
        
	    rotate.setState(false);
	    zoom.setState(false);
	    translate.setState(true);

	    root=st;
	    
	    trees=new Menu("Trees");
	    nodes=new Menu("Clicked node");
	    menuBar.add(trees);
	    menuBar.add(nodes);

	    fw=new CheckboxMenuItem("Tree in a fixed width",true);
	    fsp=new CheckboxMenuItem("Tree with a fixed spacing",false);
	    tm3d=new CheckboxMenuItem("3D tree",false);
	    freezeTree=new CheckboxMenuItem("Freeze tree",false);
	    showGoals=new CheckboxMenuItem("Show all goals",false);
	    restoreHidden=new MenuItem("Restore hidden subtrees");
	    removeRecomputed=new MenuItem("Remove recomputed nodes");
	    fw.addItemListener(this);
	    fsp.addItemListener(this);
	    tm3d.addItemListener(this);
	    freezeTree.addItemListener(this);
	    showGoals.addItemListener(this);
	    restoreHidden.addActionListener(this);
	    removeRecomputed.addActionListener(this);
	    trees.add(fw);
	    trees.add(fsp);
	    trees.add(tm3d);
	    trees.add(showGoals);
	    trees.add(removeRecomputed);
	    trees.add(restoreHidden);
	    trees.add(freezeTree);

	    showCurrentGoal=new MenuItem("Show complete goal");
	    hideSubtree=new MenuItem("Hide subtree");
	    showSubtree=new MenuItem("Show subtree only");
	    recomputeFast=new MenuItem("Recompute");
      	    recomputeSideEffects=new MenuItem("Recompute with side effects");
	    recomputeFast.addActionListener(this);
	    recomputeSideEffects.addActionListener(this);
	    hideSubtree.addActionListener(this);
	    showSubtree.addActionListener(this);
	    showCurrentGoal.addActionListener(this);
	    nodes.add(recomputeFast);
	    //	    nodes.add(recomputeSideEffects);
	    nodes.add(hideSubtree);
	    nodes.add(showSubtree);
	    nodes.add(showCurrentGoal);
	    nodes.setEnabled(false);

        windowSearchTree3D=new CLPGUIFrame3D(this);
        windowSearchTree3D.setTitle("CLPGUI Search Tree");
	    windowSearchTree3D.addWindowListener(windowSearchTree3D);
        windowSearchTree3D.show();
        this.setVisible(true);
	    clear();
	}

	void clear() {
	    center();
	    sRoot=root;
	    zoomY=0.7;
	}

	void center() {
	    super.center();
	    setO();
	    if (tm3d.getState()) {
		aZ=-pi4;
		aY=pi6;
	    }
	    else {
		aZ=0;
		aY=0;
	    }
	    initRotate();
	    repaint();
	}

	void setO() {
	    if (fsp.getState()) {
		oX=0;
		oY=0;
	    }
	    else if (fw.getState()){
		oX=width/2;
		oY=0;
	    }
	    else {
		oX=width/2;
		oY=0; //height/3;
	    }
	}

	void resized(int width, int height) {
	    super.resized(width,height);
	    setO();
	}

	public void itemStateChanged(ItemEvent e) {
	    String s=e.getItem().toString();

	    if (s.equals("Tree with a fixed spacing")) {
		fsp.setState(true);
		fw.setState(false);
		tm3d.setState(false);
		setO();
	    }
	    else if (s.equals("Tree in a fixed width")){
		fsp.setState(false);
		tm3d.setState(false);
		fw.setState(true);
		setO();
	    }
	    else if (s.equals("3D tree")){
		fsp.setState(false);
		tm3d.setState(true);
		fw.setState(false);
	    }
	    center();
	    super.itemStateChanged(e);
	    repaint();
	}

	public synchronized void actionPerformed(ActionEvent e) {
	    clicked=false;
	    nodes.setEnabled(false);
	    showCompleteGoal=false;

	    if (e.getSource()==center) {
		center();
	    } else if (e.getSource()==restoreHidden) {
		noHide=true;
		noRecompute=false;
		sRoot=root;
		cleanTree(root);
	    }
	    else if (e.getSource()==removeRecomputed) {
		noHide=false;
		noRecompute=true;
		cleanTree(root);
	    }
	    else if (highlight==null) {
		System.err.println("Action with no highlighted goal");
		return;
	    }
	    else if (e.getSource()==hideSubtree) {
		if (highlight.property.startsWith("hidden"))
		    highlight.property=highlight.property.substring(6);
		else
		    highlight.property="hidden"+highlight.property;
	    }
	    else if (e.getSource()==showSubtree) {
		sRoot=highlight;
	    }
	    else if (e.getSource()==showCurrentGoal) {
		showCompleteGoal=true;
	    }

	    repaint();
	}
	
	synchronized void cleanTree(SearchTree t) {
	    //System.err.println(t.label);
	    if (noHide & t.property.startsWith("hidden"))
		t.property=t.property.substring(6);
	    SearchTreeList l=t.children;
	    SearchTreeList p=null;
	    while (l!=null) {
		cleanTree(l.head);
		p=l;
		l=l.tail;
	    }
	}

	//////////////
	// Drawings //
	//////////////

	public synchronized void paint(Graphics g)
	{
            if (memory==null) {
                memory = createImage(width,height);
                gMem = memory.getGraphics();
                gMem.setFont(new Font("monospaced",Font.PLAIN,12));
            }

            gMem.setColor(Color.white);
            gMem.fillRect(0, 0, width, height);

            drawAll();

	        g.drawImage(memory,0,0,this);
            refresh=false;
	}
	
	synchronized void drawAll()
	{
	    if (sRoot==null) sRoot=root;
	    if (!freezeTree.getState()) {
		/*fCurrent=current;*/
		fRoot=sRoot;
	    }
	    gMem.setColor(Color.black); //gMem.setColor(translucentGray);
	    nearString="";
	    if (fsp.getState()) {
		drawCenteredStrings=false;
		double d;
		if (fRoot!=null) d=drawSubtreeFSP(fRoot,5,-20);
	    }
	    else if (fw.getState()) {
		drawCenteredStrings=true;
		if (fRoot!=null) drawSubtreeFW(fRoot,0,-20,width);
	    }
	    else {
		drawCenteredStrings=true;
		if (fRoot!=null) drawSubtree3DTM(fRoot,0,0,-20,width,height,0);
	    }

	    if (nearString!=null) {
		gMem.setColor(Color.red);
		drawString(nearString,im,jm);
	    }

	}

	// Draw a tree of fixed space between leaves, returns the width used
	synchronized double drawSubtreeFSP(SearchTree t, double y, double z)
	{
	    int i=i(0,y,z);
	    int j=j(0,y,z);
	    if (i>width || j>height) return 20;

	    drawLabel(t,0,y,z);
	    if (t.property.startsWith("hidden")) return 10;

	    SearchTreeList c=t.children;
	    if (c==null) {
		if (showGoals.getState()) return t.label.length()*charSize+10;
		else return 20;
	    }
	    else  {
		c=c.reverse();
		double yc=y;
		double zc=z-20;
		double w=0;
		while (c!=null) {
		    gMem.drawLine(i,j,i(0,yc,zc),j(0,yc,zc));
		    w=w+drawSubtreeFSP(c.head,yc,zc);
		    c=c.tail;
		    yc=y+w;
		}
		if (showGoals.getState())
		    return Math.max(w,t.label.length()*charSize+10);
		else return w;
	    }
	}

	// Draw a tree in a fixed width l with equal space for each child
	synchronized void drawSubtreeFW(SearchTree t, double y, double z, double l)
	{
	    int i=i(0,y,z);
	    int j=j(0,y,z);
	    //if (i(0,y,z)<0 || i(0,y,z)>width || j(0,y,z)>height) return;
	    if (j>height) return;

	    drawLabel(t,0,y,z);
	    if (t.property.startsWith("hidden")) return;

	    //if (l<2)
	    //drawString("...", i(0,y,z-20), j(0,y,z-20));
	    //else {
		SearchTreeList c=t.children;
		if (c==null) return;
		double yc=y+l/2;
		//double zc=z-(height+z)/3;
		double zc=z-20;
		int n=c.length();
		l=l/n;
		yc=yc-l/2;
		while (c!=null) {
		    gMem.drawLine(i,j,i(0,yc,zc),j(0,yc,zc));
		    drawSubtreeFW(c.head,yc,zc,l);
		    c=c.tail;
		    yc=yc-l;
		}
		//}
	}

	// Draw a 3D tree in a fixed rectangle k l with equal space for each child
	synchronized void drawSubtree3DTM(SearchTree t, double x, double y, double z, double k, double l, int inWidth)
	{
	    int i=i(x,y,z);
	    int j=j(x,y,z);
	    if (j>height) return;

	    drawLabel(t,x,y,z);
	    if (t.property.startsWith("hidden")) return;

       	    //if (l<2)
	    //drawString("...", i(x,y,z-20), j(x,y,z-20));
	    //else {
		SearchTreeList c=t.children;
		if (c==null) return;
		double xyc,lc;
		if (inWidth<2) {
		    xyc=y+l/2;
		    lc=l;
		}
		else {
		    xyc=x+k/2;
		    lc=k;
		}
		double zc=z-20;
		int n=c.length();
		lc=lc/n;
		xyc=xyc-lc/2;
		while (c!=null) {
		    if (inWidth<2) {
			gMem.drawLine(i,j,i(x,xyc,zc),j(x,xyc,zc));
			drawSubtree3DTM(c.head,x,xyc,zc,k,lc,inWidth+1);
		    }
		    else {
			gMem.drawLine(i,j,i(xyc,y,zc),j(xyc,y,zc));
			drawSubtree3DTM(c.head,xyc,y,zc,lc,l,(inWidth+1)%4);
		    }
		    c=c.tail;
		    xyc=xyc-lc;
		}
		//}
	}

	void drawString(String s, int i, int j) {
	    if (drawCenteredStrings) {
		int n=s.length()*charSize/2;
		gMem.drawString(s,i-n,j);
	    }
	    else gMem.drawString(s,i,j);
	}

	synchronized void drawLabel(SearchTree t, double x, double y, double z) {
	    if (showCompleteGoal & highlight==t) {
		/*if (t==fCurrent)
		    gMem.setColor(Color.blue);
		else*/
		    gMem.setColor(Color.red);
		String s="";
		SearchTree ta=highlight;
		while (ta!=root) {
		    if (!ta.label.startsWith("[")) s=ta.label+","+s;
		    ta=ta.father;
		}
		drawString(s,i(x,y,z),j(x,y,z));
	    }
	 /*   else if (t==fCurrent) {
		gMem.setColor(Color.blue);
		drawString(t.label, i(x,y,z), j(x,y,z));
	    }*/
	    else if (t==fRoot) {
		gMem.setColor(Color.green);
		drawString(t.label, i(x,y,z), j(x,y,z));
	    }
	    else if  (!clicked & near(i(x,y,z),j(x,y,z))) {
		nearString=nearString+t.label;
		if (highlight!=t) {
		    highlight=t;
		    refresh=true;
		}
	    }
	    else if (t==highlight || t.property.startsWith("show")) {
		gMem.setColor(Color.red);
		drawString(t.label, i(x,y,z), j(x,y,z));
	    }
	    else if (showGoals.getState()) {
		drawString(t.label, i(x,y,z), j(x,y,z));
	    }
	    else drawString("_", i(x,y,z), j(x,y,z));

	    if (t.property.startsWith("hidden")) {
		drawString("?", i(x,y,z-10), j(x,y,z-10));
	    }
	    else if (t.property.endsWith("success")) {
		gMem.setColor(Color.red);
		drawString("+", i(x,y,z-10), j(x,y,z-10));
	    }
	    gMem.setColor(Color.black);//gMem.setColor(translucentGray);
	}

	/////////////////////
	// Goals highlight //
	/////////////////////

	public synchronized void mouseMoved(MouseEvent e)
	{
	    im=e.getX();
	    jm=e.getY();
	    if (!clicked) highlight=null;
	    paint(getGraphics());
  	}
	public synchronized void mouseClicked(MouseEvent e)
	{
	    im=e.getX();
	    jm=e.getY();
	    if (highlight==null) {
		super.mouseClicked(e);		
	    }
	    else if (clicked) {
		clicked=false;
		nodes.setEnabled(false);
		showCompleteGoal=false;
	    }
	    else {
		clicked=true;
		nodes.setEnabled(true);
		showCompleteGoal=false;
	    }
	}
	public synchronized void mouseDragged(MouseEvent e)
	{
	    im=e.getX();
	    jm=e.getY();
	    super.mouseDragged(e);
	}
	public void mouseEntered(MouseEvent e) {
	    //clicked=false;
	    nodes.setEnabled(false);
	}

	boolean near(int i, int j) {
	    return ((i-im)*(i-im)+(j-jm)*(j-jm)<NEAR);
	}

    public void refresh(){
        refresh=true;
        this.repaint();
    }

    public Container getContentContainer(){return windowSearchTree3D;}

    public void setHighLight(SearchTree highlight){this.highlight=highlight;}

    public int getType(){return type;}

    public void reshow(){
        windowSearchTree3D.show();
    }
}

